"""
FEniCS tutorial demo program: Poisson equation with Dirichlet conditions.
Test problem is chosen to give an exact solution at all nodes of the mesh.
  -Laplace(u) = f    in the unit square
            u = u_D  on the boundary
  u_D = 1 + x^2 + 2y^2
    f = -6
"""

from __future__ import print_function
from fenics import *
import matplotlib.pyplot as plt
from mshr import *
import scipy.io
from scipy.io import savemat
parameters['linear_algebra_backend'] = "Eigen"
parameters['reorder_dofs_serial'] = False

# Create mesh and define function space
channel = Rectangle(Point(0, 0), Point(1.0, 1.0))
cylinder = Circle(Point(0.5, 0.5), 0.05)
domain = channel - cylinder
mesh = generate_mesh(domain, 60)


V = FunctionSpace(mesh, 'P', 1)

# Define boundary condition
u_D = 0

def boundary(x, on_boundary):
    return on_boundary

bc = DirichletBC(V, u_D, boundary)

# Define variational problem
u = TrialFunction(V)
v = TestFunction(V)
f = Constant(-6.0)
a = dot(grad(u), grad(v))*dx
L = f*v*dx

# Compute solution
u = Function(V)
solve(a == L, u, bc)

# Plot solution and mesh (uncomment to see the plot)
#plot(u)
#plot(mesh)
#plt.show()


# Assemble matrix
A, b = assemble_system(a, L, bc);
rows,cols,vals = as_backend_type(A).data() #Get CSR data!
A = as_backend_type(A).sparray()
b = as_backend_type(b).get_local()
savemat('Poisson_2.mat', {'A': A,'b':b})

